/*
 *  Copyright (C) 2012 Lukas F Schrittwieser
 *
 *  based on SPIProtocolAnalysis.java by Frank Kunz 
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin St, Fifth Floor, Boston, MA 02110, USA
 *
 */
 
 
 
package org.sump.analyzer.tools;
  
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
//import java.io.BufferedWriter;
import java.io.File;
//import java.io.FileWriter;
import java.util.Enumeration;
import java.lang.Float;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.JOptionPane;

import org.sump.analyzer.CapturedData;
import org.sump.analyzer.Configurable;
import org.sump.util.Properties;

public class RS232 extends Base implements Tool, Configurable
{
	/**
	 * Returns a unique string as name for this tool
	 * @return name 
	 */
	public String getName()
	{
		return "RS232 Analyzer...";
	}

	/**
	 * Performs tool initialization and creates a, yet invisible, dialogl
	 * @param frame main window's frame (needed for modal dialogs)
	 */
	public void init(Frame frame)
	{
		dialog = new RS232Dialog(frame, getName());	
	}
	
	/**
	 * This method is invoked when the tool is selected from the Tools menu.
	 * It should request any missing information using a dialog and perform the tool's actual task.
	 * @param data currently displayed captured data
	 * @return new <code>CapturedData</code> if provided data has been altered or <code>null</code> otherwise
	 */
	public CapturedData process(CapturedData data)
	{
		dialog.showDialog(data, -1); // we don't have a channel here -> use -1 as impossible value
		return null;	// we never modify the captured data
	}

	/**
	 * This method is invoked when the tool is selected from the context menu after right clicking someplace in the diagram.
	 * It should request any missing information using a dialog and perform the tool's actual task.
	 * @param data currently displayed captured data
	 * @param group channel group at the mouse position where the menu was openend
	 * @param channel number of channel at the mouse position where the menu was openend
	 * @param position number of sample at the mouse position where the menu was openend
	 * @return new <code>CapturedData</code> if provided data has been altered or <code>null</code> otherwise
	 */
	public CapturedData process(CapturedData data, int group, int channel, int position)
	{
		System.out.println("RS232 tool invoked with mouse on channel "+channel);
		dialog.showDialog(data, channel);
		return null;	// we never modify the captured data
	}
	
	/**
	 * Reads dialog settings from given properties.
	 * @param properties Properties containing dialog settings
	 */
	public void readProperties(Properties properties)
	{
		dialog.readProperties(properties);
	}

	/**
	 * Writes dialog settings to given properties.
	 * @param properties Properties where the settings are written to
	 */
	public void writeProperties(Properties properties)
	{
		dialog.writeProperties(properties);
	}
	
	/**
	 * The Dialog Class
	 * @author Frank Kunz
	 * @author Lukas F Schrittwieser
	 *
	 * The dialog class draws the basic dialog with a grid layout. The dialog
	 * consists of three main parts: A settings panel, a table panel
	 * and three buttons.
	 */
	private class RS232Dialog extends JDialog implements ActionListener
	{
		final String []	BAUD_RATES = {"1200", "2400", "4800", "9600", "14400", "19200", "28800", "38400", "57600", "115200"};
		final int 		STATE_START_BIT				= 0;
		final int		STATE_DATA					= 1;
		final int		STATE_STOP_BITS				= 2;
		final int		STATE_WAIT_FOR_LINE_IDLE 	= 3;
		
		public RS232Dialog(Frame frame, String name)
		{
			super(frame, name, false);	// create a non modal dialog
			Container pane = getContentPane();
			pane.setLayout(new GridBagLayout());
			getRootPane().setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

			/*
			 * add protocol settings elements
			 */
			JPanel panSettings = new JPanel();
			panSettings.setLayout(new GridLayout(0,2,5,5));
			panSettings.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder("Settings"),
					BorderFactory.createEmptyBorder(5, 5, 5, 5)));
						
			panSettings.add(new JLabel("Baud"));
			// present the user a list of the most common baud rates
			baudBox = new JComboBox(BAUD_RATES);
			baudBox.setEditable(true);
			selectByValue(baudBox,"115200");
			panSettings.add(baudBox);
			
			panSettings.add(new JLabel("Data Bits"));
			String [] bitarray = new String[32];
			for (int i = 0; i < bitarray.length; i++)
				bitarray[i] = new String("" + (i+1));
			dataBitsBox = new JComboBox(bitarray);
			dataBitsBox.setSelectedItem("8");
			panSettings.add(dataBitsBox);

			panSettings.add(new JLabel("Stop Bits"));
			bitarray = new String[4];
			for (int i = 0; i < bitarray.length; i++)
				bitarray[i] = new String("" + (i+2.0)/2.0);
			stopBitsBox = new JComboBox(bitarray);
			stopBitsBox.setSelectedItem("0");
			stopBitsBox.setEditable(true);
			panSettings.add(stopBitsBox);
			
			panSettings.add(new JLabel("Channel"));
			String [] charray = new String[32];
			for (int i=0; i < charray.length; i++)
				charray[i] = new String(""+i);
			channelBox = new JComboBox(charray);
			panSettings.add(channelBox);
			
			panSettings.add(new JLabel("Start at"));
			startAtBox = new JComboBox();
			startAtBox.addItem("First Sample");
			//startAtBox.setEditable(true);
			panSettings.add(startAtBox);
			
			// add the control panel to the dialog
			pane.add(panSettings, createConstraints(0, 0, 1, 1, 0.0, 0));
						
			/*
			 * add an empty output view
			 */
			JPanel panTable = new JPanel();
			panTable.setLayout(new GridLayout(1, 1, 5, 5));
			panTable.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder("Results"),
					BorderFactory.createEmptyBorder(5, 5, 5, 5)));
			String colData[][] = new String[1][5];
			colData[0][0] = new String();
			colData[0][1] = new String();
			colData[0][2] = new String();
			colData[0][3] = new String();
			colData[0][4] = new String();
			outTable = new JTable(colData, columnNames);
			panTable.add(new JScrollPane(outTable));
			pane.add(panTable, createConstraints(1, 0, 1, 2, 1.0, 1.0));
			
			/*
			 * add buttons
			 */
			JPanel panButtons = new JPanel();
			panButtons.setLayout(new GridLayout(1,0,5,0));
			JButton convert = new JButton("Analyze");
			convert.addActionListener(this);
			//add(convert, createConstraints(0, 3, 1, 1, 0, 0));
			panButtons.add(convert);
			export = new JButton("Export");
			export.setEnabled(false);
			export.addActionListener(this);
			//add(export, createConstraints(1, 3, 1, 1, 0, 0));
			panButtons.add(export);
			JButton cancel = new JButton("Close");
			cancel.addActionListener(this);
			//add(cancel, createConstraints(2, 3, 1, 1, 0, 0));
			panButtons.add(cancel);
			pane.add(panButtons, createConstraints(0, 2, 2, 1, 0, 0));
			
			pack();
			//setResizable(false);
		}
	
		/**
		 * Show the dialog to the user by setting it to visible.
		 * @param d CapturedData object containing the data samples recorded by the analyzer.
		 */
		public void showDialog(CapturedData d, int channel)
		{
			if (d == null)
			{
				System.err.println("RS232 Tool: Error in showDialog: d is null");
				return;
			}
			analysisData = d;
			// update settings user interface to accomodate captured data
			startAtBox.removeAllItems();
			startAtBox.addItem(new startAtBoxEntry("first sample", 0));
			if (analysisData.hasTriggerData())
				startAtBox.addItem(new startAtBoxEntry("Trigger", analysisData.triggerPosition));
			int i=0;
			for (Enumeration<Integer> e = analysisData.getMarkers(); e.hasMoreElements();)
			{
				i++;
				Integer m = e.nextElement();
				startAtBox.addItem(new startAtBoxEntry("Marker "+i+" ("+m.toString()+")", m.intValue()));
			}
			
			setVisible(true);
			// if we got information about the channel select the correct one
			if ((channel >= 0) && (channel < 32))
				channelBox.setSelectedIndex(channel);	
		}
		
		/**
		 * This is the action listener for user interaction with the tools dialog.
		 * It performs the necessary actions for the buttons.
		 */		
		public void actionPerformed(ActionEvent e)
		{
			if (e.getActionCommand().equals("Analyze"))
			{
				float baud = -1;
				int dataBits = -1;
				float stopBits = -1;
				int channel = -1;
				int startAt = 0;
				// read the user selections
				try
				{
					baud = Float.valueOf((String)(baudBox.getSelectedItem())).floatValue();
					dataBits = dataBitsBox.getSelectedIndex()+1;	// Note: The value "1" has index 0 -> add 1
					stopBits = Float.valueOf((String)(stopBitsBox.getSelectedItem())).floatValue();
					channel = channelBox.getSelectedIndex();
					// the sample the user whishes to start at is stored in the item of the drop down list (see inner class)
					startAt = ((startAtBoxEntry)startAtBox.getSelectedItem()).getIndex();
					if (startAt < 0) // just to be sure
						startAt = 0;
				}
				catch (NumberFormatException ex)
				{
					JOptionPane.showMessageDialog(this, "Can't read your input. Java says: "+ex, "Error Parsing Input",JOptionPane.ERROR_MESSAGE);
					return;
				}
				if (baud <= 0.0)
				{
					JOptionPane.showMessageDialog(this, "A baud rate of zero and below is not possible","Error Parsing Input",JOptionPane.ERROR_MESSAGE);
					return;
				}
				
				// create a new data model for the decoded data
				decodedDataModel = new CsvExportableTableModel(columnNames, 0);
				outTable.setModel(decodedDataModel);	// register the model where decoded data is stored
				// now call the actual analyzer
				decode(baud, channel, dataBits, stopBits, startAt);
				export.setEnabled(true);
			} 
			else if (e.getActionCommand().equals("Close"))
			{
				// clear table model to save memory
				setVisible(false);
				decodedDataModel = new CsvExportableTableModel(columnNames, 0);
				outTable.setModel(decodedDataModel);
				export.setEnabled(false);
			}
			else if (e.getActionCommand().equals("Export"))
			{
				JFileChooser fileChooser;
				fileChooser = new JFileChooser();
				fileChooser.addChoosableFileFilter((FileFilter) new CSVFilter());
				
				if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
				{
					File file = fileChooser.getSelectedFile();
					decodedDataModel.exportToCsv((Frame)getOwner(), file);
					
					//storeToFile(file);
				}
			}
		}
		
		/**
		 * analyze the data set stored in analysisData with the given settings
		 */
		private void decode(float baudRate, int channel, int numDataBits, float numStopBits, int startAt)
		{
			int ind;				// pointer in the data to analyze
			int startOfFrame = 0;	// pointer to sampe at wich the start bit begins (first falling edge)
			int dataFrame = 0;			// used to assemble the data stored in one frame
			int bitCnt = 0;			// used to count the number of bits received in the frame
			float samplesPerBit = analysisData.rate / baudRate; // note: use a float as this number might not be an integer
			int state = STATE_WAIT_FOR_LINE_IDLE;	// first we search for a high on the line
			int bit = 0;			// current line status
			int lastBit = 0;		// previous line status			
			float tolerance = 0.05F;
			int lastToggle = -1;		// index at which the last signal level change occured
			
			System.out.println("starting decode at "+startAt);
			System.out.println("samples per bit: "+samplesPerBit);
			
			if (samplesPerBit < 1.0)
			{
				JOptionPane.showMessageDialog(this, "The selected baud rate yields a resolution of "+samplesPerBit+" samples per bit. This is not enough to decode the data. Sample with a higher rate or lower the baud rate.","Decoder Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			
			assert(analysisData != null);
			assert(numDataBits > 0);
			assert(numDataBits <= 32);
			assert(baudRate > 0.0F);
			assert(channel >= 0);
			assert(channel < 32);
			assert(numStopBits > 0);
			
			// we process all samples in a loop. A simple state machine is used to decode the data
			for (ind=startAt; ind<analysisData.values.length; ind++)
			{
				// extract the bit of the channel we analyze
				lastBit = bit;	// save old line state
				bit = (analysisData.values[ind]>>channel) & 0x01;
				// check that the duration of a pulse is at least the length one bit
				if (lastBit != bit)
				{
					// the line toggled, check time between this and the previous toggling
					if ((lastToggle>=0) && ((ind-lastToggle)<(samplesPerBit*(1-tolerance))))
					{
						Object [] res = new Object[5];
						res[0] = indexToTime(ind);
						res[1] = null; 
						res[2] = null;
						res[3] = null;
						res[4] = "Pulse duration shorter than one bit time.";
						decodedDataModel.addRow(res);
					}
					lastToggle = ind;
				}
				
				// decoder FSM
				switch (state)
				{
				case STATE_START_BIT:
					{	// scan for a falling edge
						if (bit == 0)
						{
							startOfFrame = ind;
							state = STATE_DATA;
							dataFrame = 0;
							bitCnt = 0;		// reset bit counter, next bit is the first of this frame
						}
						break;
					}
					
				case STATE_DATA:
					{	// receive data bits of frame
						if ((ind-startOfFrame-(1.5+bitCnt)*samplesPerBit) >= 0.5)
						{
							// we are in the middle of a bit, sample now
							// default is LSB first transmission
							if (bit == 1)
								dataFrame |= (1<<bitCnt);
							bitCnt++;
							if (bitCnt >= numDataBits)
								state = STATE_STOP_BITS;	// we received all data bits, last step are the stop bits					
						}
						break;
					}
					
				case STATE_STOP_BITS:
					{
						// stop 20% before the expected end of the stop bit (margin)
						if ((ind-startOfFrame-(1+numDataBits+numStopBits)*(1-tolerance)*samplesPerBit) >= 0.5) 
						{
							// save the result
							//decodedData.add(new RS232AnalysisDataSet(startOfFrame, ind, dataFrame));
							Object [] res = new Object[5];
							res[0] = indexToTime(startOfFrame);
							if ((dataFrame >= 32) && (dataFrame <= 255))
								res[1] = "'" + ((char)dataFrame) + "'";
							else
								res[1] = "";
							res[2] = Integer.toString(dataFrame);
							res[3] = "0x" + Integer.toHexString(dataFrame);
							res[4] = null;
							decodedDataModel.addRow(res);
							state = STATE_START_BIT;
							break;
						}
						// once the end of all data bits has been reached we add a margin (20% of one bit) and then start checking the stop bit
						if ((ind-startOfFrame-(1+numDataBits)*samplesPerBit)*(1+tolerance) >= 0.5)
						{
							// check that the line is really high as we expect it
							if (bit == 0)
							{
								Object [] res = new Object[5];
								res[0] = indexToTime(ind);
								res[1] = null; 
								res[2] = null;
								res[3] = null;
								res[4] = "Stop bit is not high";
								decodedDataModel.addRow(res);
								state = STATE_WAIT_FOR_LINE_IDLE;
							}
						}						
						break;
					}
					
				case STATE_WAIT_FOR_LINE_IDLE:
					{
						if (bit == 1)
						{
							state = STATE_START_BIT;
						}
						break;
					}
				}
			}
		}
		
		/**
		 * Convert sample count to time string.
		 * @param count sample count (or index)
		 * @return string containing time information
		 */
		private String indexToTime(int count)
		{
			return analysisData.indexToTime(count, 1, 3);
		}
		
		/**
		 * Reads dialog settings from given properties.
		 * @param properties Properties containing dialog settings
		 */
		public void readProperties(Properties properties)
		{
			selectByIndex(baudBox, properties.getProperty("tools.RS232AnalysisTool.Baud"));
			selectByIndex(dataBitsBox, properties.getProperty("tools.RS232AnalysisTool.DataBits"));
			selectByIndex(stopBitsBox, properties.getProperty("tools.RS232AnalysisTool.StopBits"));
			selectByIndex(channelBox, properties.getProperty("tools.RS232AnalysisTool.Channel"));
		}
	
		/**
		 * Writes dialog settings to given properties.
		 * @param properties Properties where the settings are written to
		 */
		public void writeProperties(Properties properties)
		{
			properties.setProperty("tools.RS232AnalysisTool.Baud", (String)baudBox.getSelectedItem());
			properties.setProperty("tools.RS232AnalysisTool.DataBits", (String)dataBitsBox.getSelectedItem());
			properties.setProperty("tools.RS232AnalysisTool.StopBits", (String)stopBitsBox.getSelectedItem());
			properties.setProperty("tools.RS232AnalysisTool.Channel", (String)channelBox.getSelectedItem());
			
		}
		
		private String columnNames[] = { "Time", "ASCII", "Decimal", "Hex", "Error" };
		
		CapturedData analysisData;
		CsvExportableTableModel decodedDataModel;	// table model storing the decoder results
		// user interface objects
		JComboBox baudBox;
		JComboBox dataBitsBox;
		JComboBox stopBitsBox;
		JComboBox channelBox;
		JComboBox startAtBox;
		JTable    outTable;
		JButton   export;
	}
	
	/**
	 * Inner class definig an entry in the dialogs 'start at' combo box
	 */
	private class startAtBoxEntry
	{
		public startAtBoxEntry(String userMsg, int index)
		{
			ind = index;
			msg = userMsg;
		}
		
		/**
		 * Returns the user message associated with this entry
		 */
		public String toString()
		{
			return msg;
		}
		
		public int getIndex()
		{
			return ind;	
		}
		
		int ind;
		String msg;
	}
	
	/**
	 * Inner class defining a File Filter for CSV files. 
	 * 
	 */
	private class CSVFilter extends FileFilter {
		public boolean accept(File f) {
			return (f.isDirectory() || f.getName().toLowerCase().endsWith(".csv"));
		}
		public String getDescription() {
			return ("Character sepatated Values (*.csv)");
		}
	}
	
	private static GridBagConstraints createConstraints(int x, int y, int w, int h, double wx, double wy) {
	 	GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.insets = new Insets(4, 4, 4, 4);
		gbc.gridx = x; gbc.gridy = y;
		gbc.gridwidth = w; gbc.gridheight = h;
		gbc.weightx = wx; gbc.weighty = wy;
	    return (gbc);
	}
	
	RS232Dialog dialog;	// object of the inner dialog class
}


