/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2012   Lukas Schrittwieser (LS)
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2012/12/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Implements hardware access function for the DRAM controller and the data compressor hardware.
--*
--***************************************************************************************************************/

#ifndef __DRAM_H__
#define __DRAM_H__



#include <inttypes.h>


/****************************************************************************************************************
*   D E F I N E S                                          
*/

// number of task nodes to be created, has to be bigger than the number of available eth buffers
#define NUM_TASK_NODES	5

// states of the data transfer FSM
#define ST_DRAM_IDLE	0x00
#define ST_DRAM_DMA		0x01
#define ST_CMPR_FLUSH	0x02
#define ST_CMPR_DMA		0x03
#define ST_TX_DONE		0xf0
#define ST_DRAM_ERR		0xf1
#define ST_CMPR_ERR		0xf2



/**************************************************************************************************
*   H A R D W A R E   P R O P E R T I E S                                          
*/

// we have 128MBytes of sample DRAM and one sample consists of 4 bytes (32 channels)
#define SAMPLE_MEM_LENGTH		((uint32_t)(32L*1024L*1024L))
#define SAMPLE_MEM_ADDR_MASK	(0x7ffffffL)	

#define NUM_SAMPLING_CHANNELS	32

// the base frequency of the sampling hardware in MHz
#define MAX_SAMPLING_FRQ		200



/**************************************************************************************************
*   T Y P E S                                              
*/

typedef struct task
{
	uint8_t		state;		// see defines section above for possible values
	int8_t 		bufInd;		// index of the ethernet buffer which receives the data
	uint16_t	bufSize;	// total buffer size in bytes 
//	uint16_t	bufOffset;	// number of bytes to be skipped at beginning of buffer
	uint8_t*	dest;		// pointer to buffer where data will be copied to
	uint16_t	numSamples;	// number of samples to be transfered into buffer
	uint16_t	len;		// number of bytes copied into buffer
	uint8_t		useCompr;	// compression turned on / off flag
	volatile struct task	*next;	// make it a linked list (the list is always volatile)
}  task_t;



/**************************************************************************************************
*   P R O T O T Y P E S                                             
*/

void dramInit();

void dramSetPreTrigLen(uint32_t pre);

void dramSetPostTrigLen(uint32_t post);

void dramSetSource(uint32_t offset);

uint8_t	dramRegTask(volatile task_t* newTask);

//uint8_t dramWait();

//uint8_t dramGetState();

uint32_t dramGetSamplingLen();

uint8_t dramGetIntFlag();

volatile task_t* dramGetFinishedTask();

#endif
