/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   Firmware 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/17 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Header for eth.c
--*
--***************************************************************************************************************/

#ifndef __ETH_H__
#define __ETH_H__

void 	ethInitPhy();
void 	ethInit();
void 	ethSend(uint8_t bufferNum, uint16_t length);
int8_t 	ethGetTxBuffer();
int8_t  ethAllocBuffer();
void 	ethFreeBuffer(uint8_t ind);
int8_t 	ethGetRxPacket();
void 	ethRxConsume (uint8_t ind);
void 	ethWaitUntilSent (int8_t ind);
uint8_t ethGetLinkState();

/* our MAC address */
#define OWN_MAC_ADDRESS_0	0x12	/* this byte is sent first, it is the left most in an mac address like xx:yy:... */
#define OWN_MAC_ADDRESS_1	0x34
#define OWN_MAC_ADDRESS_2	0x56
#define OWN_MAC_ADDRESS_3	0x42
#define OWN_MAC_ADDRESS_4	0x4C
#define OWN_MAC_ADDRESS_5	0x41

/* Driver Configuration options */

/* total number of ethernet packet buffers, (they have to fit into the available data sram!) */
/* Note: their location is specified in eth.c */
#define	NUM_ETH_BUFFERS		8

/* How many of these ethernet buffers should be used for reception and transmission
   (the remaining buffers are used for transmission only) 
   (the buffers with the lower addresses are used for reception) */
#define NUM_ETH_RX_BUFFERS	3

/* buffer status flags */
#define BUF_FREE	0
/* buffer should be sent by the mac */
#define BUF_TX		1
/* buffer has data received by the mac */
#define BUF_RX		2
/* buffer has been allocated for data transfer but is not yet ready to be sent */
#define BUF_ALLOC	4


/* these buffers are used for reception and transmission and reside in the upper memory 
   which is not seen by the compiler. Therefore we define * const here */
extern volatile uint8_t * const ethBuf[NUM_ETH_BUFFERS]; 
/* length of data in a buffer */
extern volatile uint16_t ethBufLen[NUM_ETH_BUFFERS];


/* bit defines for the configuration register of the MAC */
#define ETH_MODER	        0x00
#define ETH_INT_SOURCE	    0x04
#define ETH_INT_MASK	    0x08
#define ETH_IPGT	        0x0C
#define ETH_IPGR1	        0x10
#define ETH_IPGR2	        0x14
#define ETH_PACKETLEN	    0x18
#define ETH_COLLCONF	    0x1C
#define ETH_TX_BD_NUM	    0x20
#define ETH_CTRLMODER	    0x24
#define ETH_MIIMODER	    0x28
#define ETH_MIICOMMAND  	0x2C
#define ETH_MIIADDRESS	    0x30
#define ETH_MIITX_DATA	    0x34
#define ETH_MIIRX_DATA	    0x38
#define ETH_MIISTATUS	    0x3C
#define ETH_MAC_ADDR0	    0x40
#define ETH_MAC_ADDR1	    0x44
#define ETH_HASH0_ADR	    0x48
#define ETH_HASH1_ADR	    0x4C
#define ETH_TXCTRL	        0x50

#define ETH_TXBD0H	        0x404
#define ETH_TXBD0L	        0x400

/* define the length of one buffer descriptor in bytes */
#define ETH_BD_LENGTH		8

#define ETH_RXBD0H	        0x604	//this depends on TX_BD_NUM but this is the standard value
#define ETH_RXBD0L	        0x600	//this depends on TX_BD_NUM but this is the standard value

//MODER BITS
#define ETH_RECSMALL	    0x00010000
#define ETH_PAD		        0x00008000
#define ETH_HUGEN	        0x00004000
#define ETH_CRCEN	        0x00002000
#define ETH_DLYCRCEN	    0x00001000
#define ETH_FULLD	        0x00000400
#define ETH_EXDFREN	        0x00000200
#define ETH_NOBCKOF	        0x00000100
#define ETH_LOOPBCK	        0x00000080
#define ETH_IFG		        0x00000040
#define ETH_PRO		        0x00000020
#define ETH_IAM		        0x00000010
#define ETH_BRO		        0x00000008
#define ETH_NOPRE	        0x00000004
#define ETH_TXEN	        0x00000002
#define ETH_RXEN	        0x00000001

// CTRLMODER bits
#define ETH_TXFLOW			0x00000004
#define ETH_RXFLOW			0x00000002
#define ETH_PASSALL			0x00000001

// MIICOMMAND bits
#define ETH_WCTRLDATA		0x00000004
#define ETH_RSTAT			0x00000002
#define ETH_SCANSTAT		0x00000001

// MIISTATUS bits
#define ETH_MII_NVALID		0x00000004
#define ETH_MII_BUSY		0x00000002
#define ETH_MII_LINKFAIL	0x00000001

//INTERRUPTS BITS
#define ETH_RXC		        0x00000040
#define ETH_TXC		        0x00000020
#define ETH_BUSY	        0x00000010
#define ETH_RXE		        0x00000008
#define ETH_RXB		        0x00000004
#define ETH_TXE		        0x00000002
#define ETH_TXB		        0x00000001

//BUFFER DESCRIPTOR BITS
#define ETH_RXBD_EMPTY	    0x00008000
#define ETH_RXBD_IRQ	    0x00004000
#define ETH_RXBD_WRAP	    0x00002000
#define ETH_RXBD_CF	        0x00000100
#define ETH_RXBD_MISS	    0x00000080
#define ETH_RXBD_OR	        0x00000040
#define ETH_RXBD_IS	        0x00000020
#define ETH_RXBD_DN	        0x00000010
#define ETH_RXBD_TL	        0x00000008
#define ETH_RXBD_SF	        0x00000004
#define ETH_RXBD_CRC	    0x00000002
#define ETH_RXBD_LC	        0x00000001

#define ETH_TXBD_READY	    0x00008000
#define ETH_TXBD_IRQ	    0x00004000
#define ETH_TXBD_WRAP	    0x00002000
#define ETH_TXBD_PAD	    0x00001000
#define ETH_TXBD_CRC	    0x00000800
#define ETH_TXBD_UR	        0x00000100
#define ETH_TXBD_RL	        0x00000008
#define ETH_TXBD_LC	        0x00000004
#define ETH_TXBD_DF	        0x00000002
#define ETH_TXBD_CS	        0x00000001

#define BD_SND  ( ETH_TXBD_READY | ETH_TXBD_IRQ | ETH_TXBD_WRAP | ETH_TXBD_PAD | ETH_TXBD_CRC )
#define RX_READY  ( ETH_RXBD_EMPTY | ETH_RXBD_IRQ )
#define TX_READY ( ETH_TXBD_IRQ | ETH_TXBD_WRAP | ETH_TXBD_PAD | ETH_TXBD_CRC )

#endif
