
#include <avr/io.h>
#include <avr/interrupt.h>

#include "eth.h"
#include "main.h"

//void uart_print_str(char *);
void uart_print_long(unsigned long);

// memory address of mac configuration interface
#define ETH_BASE	0x8000

// macro to access an 32 bit config register (memory mapped)
#define REG32(x)	*((volatile uint32_t *)(x))


volatile uint8_t tx_done;
volatile uint8_t rx_done;
volatile int rx_len;
volatile unsigned char * const tx_data  = (unsigned char*)0x1000;     
volatile unsigned char * const rx_data  = (unsigned char*)0x1600;
volatile unsigned char * const tx_data2 = (unsigned char*)0x1c00;

void uart_print_str(char *p)
{
        while(*p != 0) {
                uart_putc(*p);
                p++;
        }
}

void uart_print_long(unsigned long ul)
{
  int i;
  char c;

  
  uart_print_str("0x");
  for(i=0; i<8; i++) {

  c = (char) (ul>>((7-i)*4)) & 0xf;
  if(c >= 0x0 && c<=0x9)
    c += '0';
  else
    c += 'a' - 10;
  uart_putc(c);
  }

}

void uart_print_short(unsigned long ul)
{
  int i;
  char c;
  char flag=0;

  
  uart_print_str("0x");
  for(i=0; i<8; i++) {

  c = (char) (ul>>((7-i)*4)) & 0xf;
  if(c >= 0x0 && c<=0x9)
    c += '0';
  else
    c += 'a' - 10;
  if ((c != '0') || (i==7))
    flag=1;
  if(flag)
    uart_putc(c);
  }

}



void eth_init()
{
	//TXEN & RXEN = 1; PAD & CRC = 1; FULLD = 1
	REG32(ETH_BASE + ETH_MODER) = ETH_TXEN | ETH_RXEN | ETH_PAD | ETH_CRCEN | ETH_FULLD;
	//PHY Address = 0x001
	REG32(ETH_BASE + ETH_MIIADDRESS) = 0x00000001;

	//enable all interrupts
	REG32(ETH_BASE + ETH_INT_MASK) = ETH_RXC | ETH_TXC | ETH_BUSY | ETH_RXE | ETH_RXB | ETH_TXE | ETH_TXB;

	//set MAC ADDRESS
	REG32(ETH_BASE + ETH_MAC_ADDR1) = (uint32_t)(OWN_MAC_ADDRESS >> 32);	//low word = mac address high word
	REG32(ETH_BASE + ETH_MAC_ADDR0) = (uint32_t)OWN_MAC_ADDRESS;		//mac address rest

	//configure TXBD0
	REG32(ETH_BASE + ETH_TXBD0H) = (uint16_t)tx_data;		//address used for tx_data
	REG32(ETH_BASE + ETH_TXBD0L) = TX_READY | ETH_TXBD_WRAP;	//length = 0 | PAD & CRC = 1 | IRQ & WR = 1
	
	REG32(ETH_BASE + ETH_TXBD0H + 8) = (uint16_t)tx_data2;		//address used for tx_data
	REG32(ETH_BASE + ETH_TXBD0L + 8) = TX_READY | ETH_TXBD_WRAP;	//length = 0 | PAD & CRC = 1 | IRQ & WR = 1

	//configure RXBD0
	REG32(ETH_BASE + ETH_RXBD0H) = (uint16_t)rx_data;		//address used for tx_data
	REG32(ETH_BASE + ETH_RXBD0L) = RX_READY;	//len = 0 | IRQ & WR = 1 | EMPTY = 1

	

	//erase interrupts
	REG32(ETH_BASE + ETH_INT_SOURCE) = ETH_RXC | ETH_TXC | ETH_BUSY | ETH_RXE | ETH_RXB | ETH_TXE | ETH_TXB;

    tx_done = 1;
    rx_done = 0;
    rx_len = 0;
}

void eth_send(unsigned short length)
{
    if (!tx_done)
        return;

    tx_done = 0;
	tx_data[12] = length >> 8;
	tx_data[13] = length;

	REG32(ETH_BASE + ETH_TXBD0L) = (( 0x0000FFFFL & ( ((uint32_t)length) + HDR_LEN ) ) << 16) | BD_SND | ETH_TXBD_WRAP;
}

void eth_receive()
{
    int i;
    uart_print_str("Length: \n");
    uart_print_long(rx_len - HDR_LEN - CRC_LEN);
    uart_print_str("\n");
    uart_print_str("Data: \n");
    for ( i = 0; i < rx_len - HDR_LEN - CRC_LEN; i++ )
    {
        uart_print_short(rx_data[i+HDR_LEN]);
//        uart_print_str("\n");
    }

    rx_done = 0;
    rx_len = 0;
}

//void eth_interrupt()
ISR(INT2_vect)
{
    unsigned long source = REG32(ETH_BASE + ETH_INT_SOURCE);
    switch (source)
    {
        case ETH_TXB:
            tx_done = 1;
            //erase interrupt
            REG32(ETH_BASE + ETH_INT_SOURCE) |= ETH_TXB;
            break;

        case ETH_RXB:
            rx_done = 1;
            //erase interrupt
            REG32(ETH_BASE + ETH_INT_SOURCE) |= ETH_RXB;
            //accept further data (reset RXBD to empty)
            rx_len = (REG32(ETH_BASE + ETH_RXBD0L) >> 16);
            REG32(ETH_BASE + ETH_RXBD0L) = RX_READY;	//len = 0 | IRQ & WR = 1 | EMPTY = 1
            break;

        default:
            break;
    }
}


