
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY DataMemTb IS
END DataMemTb;
 
ARCHITECTURE behavior OF DataMemTb IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT DataMem
    PORT(
         ClkxCI : IN  std_logic;
         ClkxCIB : IN  std_logic;
         RstxRI : IN  std_logic;
         Wb8CycxSI : IN  std_logic;
         Wb8StbxSI : IN  std_logic;
         Wb8WexSI : IN  std_logic;
         Wb8AdrxDI : IN  std_logic_vector(13 downto 0);
         Wb8DataxDI : IN  std_logic_vector(7 downto 0);
         Wb8DataxDO : OUT  std_logic_vector(7 downto 0);
         Wb8AckxSO : OUT  std_logic;
         Wb32CycxSI : IN  std_logic;
         Wb32StbxSI : IN  std_logic;
         Wb32WexSI : IN  std_logic;
         Wb32SelxSI : IN  std_logic_vector(3 downto 0);
         Wb32AdrxDI : IN  std_logic_vector(13 downto 2);
         Wb32DataxDI : IN  std_logic_vector(31 downto 0);
         Wb32DataxDO : OUT  std_logic_vector(31 downto 0);
         Wb32AckxSO : OUT  std_logic
        );
    END COMPONENT;
    

   --Inputs
   signal ClkxCI : std_logic := '0';
   signal ClkxCIB : std_logic := '0';
   signal RstxRI : std_logic := '1';
   signal Wb8CycxSI : std_logic := '0';
   signal Wb8StbxSI : std_logic := '0';
   signal Wb8WexSI : std_logic := '0';
   signal Wb8AdrxDI : std_logic_vector(13 downto 0) := (others => '0');
   signal Wb8DataxDI : std_logic_vector(7 downto 0) := (others => '0');
   signal Wb32CycxSI : std_logic := '0';
   signal Wb32StbxSI : std_logic := '0';
   signal Wb32WexSI : std_logic := '0';
   signal Wb32SelxSI : std_logic_vector(3 downto 0) := (others => '0');
   signal Wb32AdrxDI : std_logic_vector(13 downto 2) := (others => '0');
   signal Wb32DataxDI : std_logic_vector(31 downto 0) := (others => '0');

 	--Outputs
   signal Wb8DataxDO : std_logic_vector(7 downto 0);
   signal Wb8AckxSO : std_logic;
   signal Wb32DataxDO : std_logic_vector(31 downto 0);
   signal Wb32AckxSO : std_logic;

   -- Clock period definitions
   constant ClkxCI_period : time := 25 ns;
   
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: DataMem PORT MAP (
          ClkxCI => ClkxCI,
          ClkxCIB => ClkxCIB,
          RstxRI => RstxRI,
          Wb8CycxSI => Wb8CycxSI,
          Wb8StbxSI => Wb8StbxSI,
          Wb8WexSI => Wb8WexSI,
          Wb8AdrxDI => Wb8AdrxDI,
          Wb8DataxDI => Wb8DataxDI,
          Wb8DataxDO => Wb8DataxDO,
          Wb8AckxSO => Wb8AckxSO,
          Wb32CycxSI => Wb32CycxSI,
          Wb32StbxSI => Wb32StbxSI,
          Wb32WexSI => Wb32WexSI,
          Wb32SelxSI => Wb32SelxSI,
          Wb32AdrxDI => Wb32AdrxDI,
          Wb32DataxDI => Wb32DataxDI,
          Wb32DataxDO => Wb32DataxDO,
          Wb32AckxSO => Wb32AckxSO
        );

   -- Clock process definitions
   ClkxCI_process :process
   begin
		ClkxCI <= '0';
		wait for ClkxCI_period/2;
		ClkxCI <= '1';
		wait for ClkxCI_period/2;
   end process;
 
   ClkxCIB <= not ClkxCI;
 

   -- Stimulus process
   stim_proc: process
   begin		
      -- hold reset state for 100 ns.
      wait for 100 ns;	
	  RstxRI <= '0';
	  
	  -- write 4 bytes on the 8 bit bus
      wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '1';
	  Wb8AdrxDI <= "10000000000000";
	  Wb8DataxDI <= x"11";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  	  
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '1';
	  Wb8AdrxDI <= "10000000000001";
	  Wb8DataxDI <= x"22";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  Wb8WexSI	<= '0';
	  
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '1';
	  Wb8AdrxDI <= "10000000000010";
	  Wb8DataxDI <= x"33";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  Wb8WexSI	<= '0';

	  wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '1';
	  Wb8AdrxDI <= "10000000000011";
	  Wb8DataxDI <= x"44";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  Wb8WexSI	<= '0';
	  
	  -- read from the 32 bit bus
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb32CycxSI 	<= '1';
	  Wb32StbxSI 	<= '1';
	  Wb32SelxSI	<= "1111";
	  Wb32WexSI	 	<= '0';
	  Wb32AdrxDI 	<= "100000000000";
	  Wb32DataxDI 	<= x"00000000";
	  wait until ClkxCI'event and ClkxCI='1' and Wb32AckxSO='1';
	  Wb32CycxSI	<= '0';
	  Wb32StbxSI 	<= '0';
	  Wb32WexSI		<= '0';
	  
	  -- write to the 32 bit bus
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb32CycxSI 	<= '1';
	  Wb32StbxSI 	<= '1';
	  Wb32SelxSI	<= "0011";
	  Wb32WexSI	 	<= '1';
	  Wb32AdrxDI 	<= "100000000000";
	  Wb32DataxDI 	<= x"0000BBAA";
	  wait until ClkxCI'event and ClkxCI='1' and Wb32AckxSO='1';
	  Wb32CycxSI	<= '0';
	  Wb32StbxSI 	<= '0';
	  Wb32WexSI		<= '0';
      
	  -- read from the 8 bit bus
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '0';
	  Wb8AdrxDI <= "10000000000000";
	  Wb8DataxDI <= x"44";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  Wb8WexSI	<= '0';
	  
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '0';
	  Wb8AdrxDI <= "10000000000001";
	  Wb8DataxDI <= x"44";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  Wb8WexSI	<= '0';
	  
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '0';
	  Wb8AdrxDI <= "10000000000010";
	  Wb8DataxDI <= x"44";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  Wb8WexSI	<= '0';
	  
	  wait until ClkxCI'event and ClkxCI='1';
	  Wb8CycxSI	<= '1';
	  Wb8StbxSI <= '1';
	  Wb8WexSI	<= '0';
	  Wb8AdrxDI <= "10000000000011";
	  Wb8DataxDI <= x"44";
	  wait until ClkxCI'event and ClkxCI='1' and Wb8AckxSO='1';
	  Wb8CycxSI	<= '0';
	  Wb8StbxSI <= '0';
	  Wb8WexSI	<= '0';
	  
      wait;
   end process;

END;
