
library ieee;
use ieee.std_logic_1164.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;

entity LATop_tb is
end LATop_tb;

architecture behavior of LATop_tb is

  -- Component Declaration for the Unit Under Test (UUT)
  
  component LATop
    generic (
      IS_SIMULATION : string  := "FALSE";
      SYS_TIMER_TOP : integer := (40000000 / 100)-1);
    port(
      xtalClock      : in    std_logic;
      resetSwitch    : in    std_logic;
      exClockxCI     : in    std_logic;
      inputChxDI     : in    std_logic_vector(31 downto 0);
      testOutputxSO  : out   std_logic;
      UartTxxDO      : out   std_logic;
      UartRxxDI      : in    std_logic;
      PHY_INT        : in    std_logic;
      PHY_MDC        : out   std_logic;
      PHY_MDIO       : inout std_logic;
      PHY_RESET      : out   std_logic;
      PHY_RXCLK      : in    std_logic;
      PHY_RXCTL_RXDV : in    std_logic;
      PHY_RXD        : in    std_logic_vector(7 downto 0);
      PHY_RXER       : in    std_logic;
      PHY_TXCLK      : in    std_logic;
      PHY_TXCTL_TXEN : out   std_logic;
      PHY_TXC_GTXCLK : in    std_logic;
      PHY_TXD        : out   std_logic_vector(7 downto 0);
      PHY_TXER       : out   std_logic;
      MemDQxDIO      : inout std_logic_vector(15 downto 0);
      MemAdrxDO      : out   std_logic_vector(12 downto 0);
      MemBaxDO       : out   std_logic_vector(2 downto 0);
      MemRasxEO      : out   std_logic;
      MemCasxEO      : out   std_logic;
      MemWexEO       : out   std_logic;
      MemOdtxO       : out   std_logic;
      MemCkexEO      : out   std_logic;
      MemClkxCO      : out   std_logic;
      MemClkxCOB     : out   std_logic;
      MemLDqsxEO     : inout std_logic;
      MemLDqsxEOB    : inout std_logic;
      MemUDqsxEO     : inout std_logic;
      MemUDqsxEOB    : inout std_logic;
      MemLdmxEO      : out   std_logic;
      MemUdmxEO      : out   std_logic;
      RzqxIO         : inout std_logic;
      ZioxIO         : inout std_logic;
      SpiSckxSO      : out   std_logic;
      SpiMosixDO     : out   std_logic;
      SpiMisoxDI     : in    std_logic;
      SpiSSxSO       : out   std_logic_vector(2 downto 0);
      SwitchxDI      : in    std_logic_vector(3 downto 0);
      ButtonxDI      : in    std_logic_vector(3 downto 0);
      LedxDO         : out   std_logic_vector(7 downto 0)
      );
  end component;


  component ddr2
    port (
      ck      : in    std_logic;
      ck_n    : in    std_logic;
      cke     : in    std_logic;
      cs_n    : in    std_logic;
      ras_n   : in    std_logic;
      cas_n   : in    std_logic;
      we_n    : in    std_logic;
      dm_rdqs : inout std_logic_vector(1 downto 0);
      ba      : in    std_logic_vector(2 downto 0);
      addr    : in    std_logic_vector(12 downto 0);
      dq      : inout std_logic_vector(15 downto 0);
      dqs     : inout std_logic_vector(1 downto 0);
      dqs_n   : inout std_logic_vector(1 downto 0);
      rdqs_n  : out   std_logic_vector(1 downto 0);
      odt     : in    std_logic
      );
  end component;

  -- testbench simulating a ethernet phy supplied along with the mac
  component eth_phy is  -- // This PHY model simulate simplified Intel LXT971A PHY
    port (
      -- COMMON
      m_rst_n_i : in    std_logic;
      -- MAC TX
      mtx_clk_o : out   std_logic;
      mtxd_i    : in    std_logic_vector(3 downto 0);
      mtxen_i   : in    std_logic;
      mtxerr_i  : in    std_logic;
      -- MAC RX
      mrx_clk_o : out   std_logic;
      mrxd_o    : out   std_logic_vector(3 downto 0);
      mrxdv_o   : out   std_logic;
      mrxerr_o  : out   std_logic;
      -- common mii
      mcoll_o   : out   std_logic;
      mcrs_o    : out   std_logic;
      -- MIIM
      mdc_i     : in    std_logic;
      md_io     : inout std_logic;
      -- SYSTEM
      phy_log   : in    std_logic_vector(31 downto 0));
  end component;

  --Inputs
  signal xtalClock      : std_logic                     := '0';
  signal resetSwitch    : std_logic                     := '1';
  signal exClockxCI     : std_logic                     := '0';
  signal inputChxDI     : std_logic_vector(31 downto 0) := (others => '0');
  signal UartRxxDI      : std_logic                     := '0';
  signal PHY_INT        : std_logic                     := '0';
  signal PHY_RXCLK      : std_logic                     := '0';
  signal PHY_RXCTL_RXDV : std_logic                     := '0';
  signal PHY_RXD        : std_logic_vector(7 downto 0)  := (others => '0');
  signal PHY_RXER       : std_logic                     := '0';
  signal PHY_TXCLK      : std_logic                     := '0';
  signal PHY_TXC_GTXCLK : std_logic                     := '0';
  signal SwitchxDI      : std_logic_vector(3 downto 0)  := (others => '1');
  signal ButtonxDI      : std_logic_vector(3x downto 0)  := (others => '1');
  signal SpiMisoxDI     : std_logic                     := '0';

  --BiDirs
  signal PHY_MDIO    : std_logic;
  signal MemDQxDIO   : std_logic_vector(15 downto 0);
  signal MemLDqsxEO  : std_logic;
  signal MemLDqsxEOB : std_logic;
  signal MemUDqsxEO  : std_logic;
  signal MemUDqsxEOB : std_logic;
  signal RzqxIO      : std_logic;
  signal ZioxIO      : std_logic;

  --Outputs
  signal testOutputxSO  : std_logic;
  signal UartTxxDO      : std_logic;
  signal PHY_MDC        : std_logic;
  signal PHY_RESET      : std_logic;
  signal PHY_TXCTL_TXEN : std_logic;
  signal PHY_TXD        : std_logic_vector(7 downto 0);
  signal PHY_TXER       : std_logic;
  signal MemAdrxDO      : std_logic_vector(12 downto 0);
  signal MemBaxDO       : std_logic_vector(2 downto 0);
  signal MemRasxEO      : std_logic;
  signal MemCasxEO      : std_logic;
  signal MemWexEO       : std_logic;
  signal MemOdtxO       : std_logic;
  signal MemCkexEO      : std_logic;
  signal MemClkxCO      : std_logic;
  signal MemClkxCOB     : std_logic;
  signal MemLdmxEO      : std_logic;
  signal MemUdmxEO      : std_logic;
  signal SpiSckxSO      : std_logic;
  signal SpiMosixDO     : std_logic;
  signal SpiSSxSO       : std_logic_vector(2 downto 0);
  signal LedxDO         : std_logic_vector(7 downto 0);

  -- signals needed to connect the Micron DDR2 model, unused for Elpida model
  signal dm    : std_logic_vector(1 downto 0);
  signal dqs   : std_logic_vector(1 downto 0);
  signal dqs_n : std_logic_vector(1 downto 0);

  -- Clock period definitions
  constant BOARD_CLOCK_PERIOD : time := 10 ns;
  
begin

  -- Instantiate the Unit Under Test (UUT)
  uut : LATop
    generic map (
      IS_SIMULATION => "TRUE",
      SYS_TIMER_TOP => 100)
    port map (
      xtalClock      => xtalClock,
      resetSwitch    => resetSwitch,
      exClockxCI     => exClockxCI,
      inputChxDI     => inputChxDI,
      testOutputxSO  => testOutputxSO,
      UartTxxDO      => UartTxxDO,
      UartRxxDI      => UartRxxDI,
      PHY_INT        => PHY_INT,
      PHY_MDC        => PHY_MDC,
      PHY_MDIO       => PHY_MDIO,
      PHY_RESET      => PHY_RESET,
      PHY_RXCLK      => PHY_RXCLK,
      PHY_RXCTL_RXDV => PHY_RXCTL_RXDV,
      PHY_RXD        => PHY_RXD,
      PHY_RXER       => PHY_RXER,
      PHY_TXCLK      => PHY_TXCLK,
      PHY_TXCTL_TXEN => PHY_TXCTL_TXEN,
      PHY_TXC_GTXCLK => PHY_TXC_GTXCLK,
      PHY_TXD        => PHY_TXD,
      PHY_TXER       => PHY_TXER,
      MemDQxDIO      => MemDQxDIO,
      MemAdrxDO      => MemAdrxDO,
      MemBaxDO       => MemBaxDO,
      MemRasxEO      => MemRasxEO,
      MemCasxEO      => MemCasxEO,
      MemWexEO       => MemWexEO,
      MemOdtxO       => MemOdtxO,
      MemCkexEO      => MemCkexEO,
      MemClkxCO      => MemClkxCO,
      MemClkxCOB     => MemClkxCOB,
      MemLDqsxEO     => MemLDqsxEO,
      MemLDqsxEOB    => MemLDqsxEOB,
      MemUDqsxEO     => MemUDqsxEO,
      MemUDqsxEOB    => MemUDqsxEOB,
      MemLdmxEO      => MemLdmxEO,
      MemUdmxEO      => MemUdmxEO,
      RzqxIO         => RzqxIO,
      ZioxIO         => ZioxIO,
      SpiSckxSO      => SpiSckxSO,
      SpiMosixDO     => SpiMosixDO,
      SpiMisoxDI     => SpiMisoxDI,
      SpiSSxSO       => SpiSSxSO,
      SwitchxDI      => SwitchxDI,
      ButtonxDI      => ButtonxDI,
      LedxDO         => LedxDO
      );

  -- board clock process 
  boardClkPrcs : process
  begin
    xtalClock <= '1';
    wait for BOARD_CLOCK_PERIOD / 2;
    xtalClock <= '0';
    wait for BOARD_CLOCK_PERIOD / 2;
  end process;


  -- Stimulus process
  stim_proc : process
  begin
    -- hold reset state
    wait for 10 ns;
    resetSwitch <= '0';
    -- Note: we have no stimulus here, the uC firmware loaded into the
    -- program memory is the stimulus as we want to see how firmware
    -- and hardware interact.      
    wait;
  end process;


  -- ddr2 memory model from micron
  dm    <= MemUdmxEO & MemLdmxEO;
  dqs   <= MemUDqsxEO & MemLDqsxEO;
  dqs_n <= MemUDqsxEOB & MemLDqsxEOB;
  ddr2ram : ddr2
    port map (
      -- ports of Micron ddr2 memory model
      ck      => MemClkxCO,
      ck_n    => MemClkxCOB,
      cke     => MemCkexEO,
      cs_n    => '0',
      ras_n   => MemRasxEO,
      cas_n   => MemCasxEO,
      we_n    => MemWexEO,
      dm_rdqs => dm,
      ba      => MemBaxDO,
      addr    => MemAdrxDO,
      dq      => MemDQxDIO,
      dqs     => dqs,
      dqs_n   => dqs_n,
      rdqs_n  => open,
      odt     => MemOdtxO
      );


  -- phy model (provided by MAC test bench, not a real HW model)
  eth_phy_inst : eth_phy
    port map (
      -- COMMON
      m_rst_n_i => PHY_RESET,
      -- MAC TX
      mtx_clk_o => PHY_TXCLK,
      mtxd_i    => PHY_TXD(3 downto 0),
      mtxen_i   => PHY_TXCTL_TXEN,
      mtxerr_i  => PHY_TXER,
      -- MAC RX
      mrx_clk_o => PHY_RXCLK,
      mrxd_o    => PHY_RXD(3 downto 0),
      mrxdv_o   => PHY_RXCTL_RXDV,
      mrxerr_o  => PHY_RXER,
      -- common mii
      mcoll_o   => open,
      mcrs_o    => open,
      -- MIIM
      mdc_i     => PHY_MDC,
      md_io     => PHY_MDIO,
      -- SYSTEM
      phy_log   => x"00000000"
      );

end;
