--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* Test bench for the data bus section of the SOC.
--*
--***************************************************************************************************************
library ieee;
use ieee.std_logic_1164.all;


entity dataBusTb is
end dataBusTb;

architecture TB of dataBusTb is

  constant SIMULATE_DRAM : boolean := false;  -- enable or disable simulation of dram controller module
  constant SIMULATE_MAC  : boolean := false;  -- enable or disable simulation of the ethernet controller (MAC)

  component dataBus
    generic (
      IS_SIMULATION : string;
      SIMULATE_DRAM : boolean;
      SIMULATE_MAC  : boolean);
    port (
      RstButtonxR     : in    std_logic;
      BoardClkxCI     : in    std_logic;
      LaCoreClkxCI    : in    std_logic;
      LaCoreClkxCBI   : in    std_logic;
      LaCoreRstxRI    : in    std_logic;
      RefClkxCO       : out   std_logic;
      McbReadyxSO     : out   std_logic;
      ClkxCI          : in    std_logic;
      ClkxCIB         : in    std_logic;
      RstxRI          : in    std_logic;
      CpuCycxSI       : in    std_logic;
      CpuStbxSI       : in    std_logic;
      CpuWexSI        : in    std_logic;
      CpuAdrxDI       : in    std_logic_vector(15 downto 0);
      CpuDatxDI       : in    std_logic_vector(7 downto 0);
      CpuDatxDO       : out   std_logic_vector(7 downto 0);
      CpuAckxSO       : out   std_logic;
      CpuErrxSO       : out   std_logic;
      CpuRtyxSO       : out   std_logic;
      PhyTxClkxCI     : in    std_logic;
      PhyTxdxDO       : out   std_logic_vector(3 downto 0);
      PhyTxEnxSO      : out   std_logic;
      PhyTxErrxSO     : out   std_logic;
      PhyRxClkxCI     : in    std_logic;
      PhyRxdxDI       : in    std_logic_vector(3 downto 0);
      PhyRxDvxSI      : in    std_logic;
      PhyRxErrxSI     : in    std_logic;
      PhyColxSI       : in    std_logic;
      PhyCrsxSI       : in    std_logic;
      PhyMdcxSO       : out   std_logic;
      PhyMdioxDI      : in    std_logic;
      PhyMdioxDO      : out   std_logic;
      PhyMdioEnxSO    : out   std_logic;
      MemDQxDIO       : inout std_logic_vector(15 downto 0);
      MemAdrxDO       : out   std_logic_vector(12 downto 0);
      MemBaxDO        : out   std_logic_vector(2 downto 0);
      MemRasxEO       : out   std_logic;
      MemCasxEO       : out   std_logic;
      MemWexEO        : out   std_logic;
      MemOdtxO        : out   std_logic;
      MemCkexEO       : out   std_logic;
      MemClkxCO       : out   std_logic;
      MemClkxCOB      : out   std_logic;
      MemLDqsxEO      : inout std_logic;
      MemLDqsxEOB     : inout std_logic;
      MemUDqsxEO      : inout std_logic;
      MemUDqsxEOB     : inout std_logic;
      MemLdmxEO       : out   std_logic;
      MemUdmxEO       : out   std_logic;
      RzqxIO          : inout std_logic;
      ZioxIO          : inout std_logic;
      exClockxCI      : in    std_logic;
      inputChxDI      : in    std_logic_vector(31 downto 0);
      testOutputxSO   : out   std_logic;
      LaCoreIRQxSO    : out   std_logic;
      LaCoreIntAckxSI : in    std_logic;
      EthIntxSO       : out   std_logic;
      CmpIntxSO       : out   std_logic);
  end component;


  -- testbench simulating a ethernet phy supplied along with the mac
  component eth_phy is  -- // This PHY model simulate simplified Intel LXT971A PHY
    port (
      -- COMMON
      m_rst_n_i : in    std_logic;
      -- MAC TX
      mtx_clk_o : out   std_logic;
      mtxd_i    : in    std_logic_vector(3 downto 0);
      mtxen_i   : in    std_logic;
      mtxerr_i  : in    std_logic;
      -- MAC RX
      mrx_clk_o : out   std_logic;
      mrxd_o    : out   std_logic_vector(3 downto 0);
      mrxdv_o   : out   std_logic;
      mrxerr_o  : out   std_logic;
      -- common mii
      mcoll_o   : out   std_logic;
      mcrs_o    : out   std_logic;
      -- MIIM
      mdc_i     : in    std_logic;
      md_io     : inout std_logic;
      -- SYSTEM
      phy_log   : in    std_logic_vector(31 downto 0));
  end component;

  --Inputs
  signal LaCoreClkxCI    : std_logic                     := '0';
  signal LaCoreClkxCBI   : std_logic                     := '1';
  signal ClkxCI          : std_logic                     := '0';
  signal ClkxCIB         : std_logic                     := '0';
  signal RstxRI          : std_logic                     := '1';
  signal CpuCycxSI       : std_logic                     := '0';
  signal CpuStbxSI       : std_logic                     := '0';
  signal CpuWexSI        : std_logic                     := '0';
  signal CpuAdrxDI       : std_logic_vector(15 downto 0) := (others => '0');
  signal CpuDatxDI       : std_logic_vector(7 downto 0)  := (others => '0');
  signal PhyTxClkxCI     : std_logic                     := '0';
  signal PhyRxClkxCI     : std_logic                     := '0';
  signal PhyRxdxDI       : std_logic_vector(3 downto 0)  := (others => '0');
  signal PhyRxDvxSI      : std_logic                     := '0';
  signal PhyRxErrxSI     : std_logic                     := '0';
  signal PhyColxSI       : std_logic                     := '0';
  signal PhyCrsxSI       : std_logic                     := '0';
  signal PhyMdioxDI      : std_logic                     := '0';
  signal exClockxCI      : std_logic                     := '0';
  signal inputChxDI      : std_logic_vector(31 downto 0) := (others => '0');
  signal LaCoreIntAckxSI : std_logic                     := '0';

  --Outputs
  signal CpuDatxDO     : std_logic_vector(7 downto 0);
  signal CpuAckxSO     : std_logic;
  signal CpuErrxSO     : std_logic;
  signal CpuRtyxSO     : std_logic;
  signal PhyTxdxDO     : std_logic_vector(3 downto 0);
  signal PhyTxEnxSO    : std_logic;
  signal PhyTxErrxSO   : std_logic;
  signal PhyMdcxSO     : std_logic;
  signal PhyMdioxDO    : std_logic;
  signal PhyMdioEnxSO  : std_logic;
  signal EthIntxSO     : std_logic;
  signal testOutputxSO : std_logic;
  signal LaCoreIRQxSO  : std_logic;
  signal CmpIntxSO     : std_logic;
  signal MemAdrxDO     : std_logic_vector(12 downto 0);
  signal MemBaxDO      : std_logic_vector(2 downto 0);
  signal MemRasxEO     : std_logic;
  signal MemCasxEO     : std_logic;
  signal MemWexEO      : std_logic;
  signal MemOdtxO      : std_logic;
  signal MemCkexEO     : std_logic;
  signal MemClkxCO     : std_logic;
  signal MemClkxCOB    : std_logic;
  signal MemLdmxEO     : std_logic;
  signal MemUdmxEO     : std_logic;

  --inouts 
  signal MemLDqsxEO  : std_logic;
  signal MemLDqsxEOB : std_logic;
  signal MemUDqsxEO  : std_logic;
  signal MemUDqsxEOB : std_logic;
  signal MemDQxDIO   : std_logic_vector(15 downto 0);
  signal RzqxIO      : std_logic;
  signal ZioxIO      : std_logic;
  signal PhyMdioxD   : std_logic;
  signal PhyLogxD    : std_logic_vector(31 downto 0) := x"00000000";


  -- Clock period definitions
  constant ClkxCI_period       : time := 25 ns;
  constant LaCoreClkxCI_period : time := 5 ns;

  signal WbData  : std_logic_vector(31 downto 0) := (others => '0');
  signal WbAdr   : std_logic_vector(15 downto 0) := (others => '0');
  signal WbStart : std_logic                     := '0';
  signal WbDone  : std_logic                     := '0';

  signal OwnMacAdr : std_logic_vector(47 downto 0) := x"12345678ABCD";
  
begin

  -- Instantiate the Unit Under Test (UUT)
  uut : dataBus
    generic map (
      IS_SIMULATION => "TRUE",
      SIMULATE_DRAM => SIMULATE_DRAM,
      SIMULATE_MAC  => SIMULATE_MAC)
    port map (
      RstButtonxR     => '0',
      BoardClkxCI     => '0',
      LaCoreClkxCI    => LaCoreClkxCI,
      LaCoreClkxCBI   => LaCoreClkxCBI,
      LaCoreRstxRI    => RstxRI,
      RefClkxCO       => open,
      McbReadyxSO     => open,
      ClkxCI          => ClkxCI,
      ClkxCIB         => ClkxCIB,
      RstxRI          => RstxRI,
      CpuCycxSI       => CpuCycxSI,
      CpuStbxSI       => CpuStbxSI,
      CpuWexSI        => CpuWexSI,
      CpuAdrxDI       => CpuAdrxDI,
      CpuDatxDI       => CpuDatxDI,
      CpuDatxDO       => CpuDatxDO,
      CpuAckxSO       => CpuAckxSO,
      CpuErrxSO       => CpuErrxSO,
      CpuRtyxSO       => CpuRtyxSO,
      PhyTxClkxCI     => PhyTxClkxCI,
      PhyTxdxDO       => PhyTxdxDO,
      PhyTxEnxSO      => PhyTxEnxSO,
      PhyTxErrxSO     => PhyTxErrxSO,
      PhyRxClkxCI     => PhyRxClkxCI,
      PhyRxdxDI       => PhyRxdxDI,
      PhyRxDvxSI      => PhyRxDvxSI,
      PhyRxErrxSI     => PhyRxErrxSI,
      PhyColxSI       => PhyColxSI,
      PhyCrsxSI       => PhyCrsxSI,
      PhyMdcxSO       => PhyMdcxSO,
      PhyMdioxDI      => PhyMdioxDI,
      PhyMdioxDO      => PhyMdioxDO,
      PhyMdioEnxSO    => PhyMdioEnxSO,
      MemDQxDIO       => MemDQxDIO,
      MemAdrxDO       => MemAdrxDO,
      MemBaxDO        => MemBaxDO,
      MemRasxEO       => MemRasxEO,
      MemCasxEO       => MemCasxEO,
      MemWexEO        => MemWexEO,
      MemOdtxO        => MemOdtxO,
      MemCkexEO       => MemCkexEO,
      MemClkxCO       => MemClkxCO,
      MemClkxCOB      => MemClkxCOB,
      MemLDqsxEO      => MemLDqsxEO,
      MemLDqsxEOB     => MemLDqsxEOB,
      MemUDqsxEO      => MemUDqsxEO,
      MemUDqsxEOB     => MemUDqsxEOB,
      MemLdmxEO       => MemLdmxEO,
      MemUdmxEO       => MemUdmxEO,
      RzqxIO          => RzqxIO,
      ZioxIO          => ZioxIO,
      exClockxCI      => ExClockxCI,
      inputChxDI      => inputChxDI,
      testOutputxSO   => testOutputxSO,
      LaCoreIRQxSO    => LaCoreIRQxSO,
      LaCoreIntAckxSI => LaCoreIntAckxSI,
      EthIntxSO       => EthIntxSO,
      CmpIntxSO       => CmpIntxSO
      );

  -- Clock process definitions
  ClkxCI_process : process
  begin
    ClkxCI <= '0';
    wait for ClkxCI_period/2;
    ClkxCI <= '1';
    wait for ClkxCI_period/2;
  end process;
  ClkxCIB <= not ClkxCI;

  LaCoreClkxCI  <= not LaCoreClkxCI after LaCoreClkxCI_period/2;
  LaCoreClkxCBI <= not LaCoreClkxCI;


  -- Stimulus process
  stim_proc : process
  begin
    -- hold reset state for 100 ns.
    wait for 100 ns;
    RstxRI <= '0';



--              wait until ClkxCI'event and ClkxCI='1';
--              -- write mode register
--              CpuWexSI <= '1';
--              WbData  <= x"00005403";
--              WbAdr   <= x"8000";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';
--              -- set mac address
--              WbData  <= OwnMacAdr(47 downto 16);
--              WbAdr   <= x"8040";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';
--              WbData  <= x"0000" & OwnMacAdr(15 downto 0);
--              WbAdr   <= x"8044";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';
--              -- Note: we use sram starting at address 0 as data buffer for a dummy package
--              -- set dest and src mac addresses
--              WbData  <= x"ffffffff"; -- first sram address holding the data package
--              WbAdr   <= x"0000";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';
--              WbData  <= x"0201ffff"; 
--              WbAdr   <= x"0004";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';
--              WbData  <= x"06050403"; 
--              WbAdr   <= x"0008";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';
--              -- add some dummy data into the package, just for testing
--              WbData  <= x"00abcdef"; 
--              WbAdr   <= x"000c";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';

--              -- set tx buffer descriptor
--              WbData  <= x"00000000"; -- first sram address holding the data package
--              WbAdr   <= x"8404";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              wait until ClkxCI'event and ClkxCI='1';
--              WbData  <= x"0010b800";         -- mark buffer descriptor as read for transmission
--              WbAdr   <= x"8400";     
--              WbStart <= '1';
--              wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--              WbStart <= '0';
--              CpuWexSI <= '0';
--              wait until ClkxCI'event and ClkxCI='1';

--              -- read the buffer descriptor to see when the package was sent completely
--              loop
--                      WbData  <= x"00000000";
--                      WbAdr   <= x"8400";     
--                      WbStart <= '1';
--                      wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
--                      WbStart <= '0';
--                      wait until ClkxCI'event and ClkxCI='1';
--                      wait for 10 us;
--              end loop;

    -- test the data compressor

    -- reset core 
    CpuWexSI <= '1';
    WbData   <= x"00000001";
    WbAdr    <= x"c01c";
    WbStart  <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart  <= '0';
    wait until ClkxCI'event and ClkxCI = '1';   
    
    -- set inc dest addr flag 
    CpuWexSI <= '1';
    WbData   <= x"00000100";
    WbAdr    <= x"c004";
    WbStart  <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart  <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- setup dma destination register
    WbData  <= x"00002000";             -- start at address 4096
    WbAdr   <= x"c010";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- setup dma len
    WbData  <= x"00000020";
    WbAdr   <= x"c014";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- send some data to compressor
    WbData  <= x"00000000";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- send some data to compressor
    WbData  <= x"00000000";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

     -- send some data to compressor
    WbData  <= x"00000000";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

     -- send some data to compressor
    WbData  <= x"00000000";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

     -- send some data to compressor
    WbData  <= x"00000000";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

     -- send some data to compressor
    WbData  <= x"00000000";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- flush core
    WbData  <= x"00000002";
    WbAdr   <= x"c01c";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    CpuWexSI <= '0';

    wait for ClkxCI_period*100;

    -- read dma destination register
    WbData  <= x"00000000";             
    WbAdr   <= x"c010";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';
    
    -- reset core 
    CpuWexSI <= '1';
    WbData   <= x"00000001";
    WbAdr    <= x"c01c";
    WbStart  <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart  <= '0';
    wait until ClkxCI'event and ClkxCI = '1';   

    -- set inc dest addr flag 
    CpuWexSI <= '1';
    WbData   <= x"00000100";
    WbAdr    <= x"c004";
    WbStart  <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart  <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- setup dma len
    WbData  <= x"00000020";
    WbAdr   <= x"c014";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- send some data to compressor
    WbData  <= x"40302010";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- send some data to compressor
    WbData  <= x"40302010";
    WbAdr   <= x"c000";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    -- flush core
    WbData  <= x"00000002";
    WbAdr   <= x"c01c";
    WbStart <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbDone = '1';
    WbStart <= '0';
    wait until ClkxCI'event and ClkxCI = '1';

    wait;
  end process;

  genPhy : if SIMULATE_MAC = true generate

    -- phy model
    eth_phy_inst : eth_phy
      port map (
        -- COMMON
        m_rst_n_i => RstxRI,
        -- MAC TX
        mtx_clk_o => PhyTxClkxCI,
        mtxd_i    => PhyTxdxDO,
        mtxen_i   => PhyTxEnxSO,
        mtxerr_i  => PhyTxErrxSO,
        -- MAC RX
        mrx_clk_o => PhyRxClkxCI,
        mrxd_o    => PhyRxdxDI,
        mrxdv_o   => PhyRxDvxSI,
        mrxerr_o  => PhyRxErrxSI,
        -- common mii
        mcoll_o   => PhyColxSI,
        mcrs_o    => PhyCrsxSI,
        -- MIIM
        mdc_i     => PhyMdcxSO,
        md_io     => PhyMdioxD,
        -- SYSTEM
        phy_log   => PhyLogxD
        );

-- tristate buffer for Mdio pin of phy
    PhyMdioxDI <= PhyMdioxD;
    process (PhyMdioxDO, PhyMdioEnxSO)
    begin
      PhyMdioxD <= 'Z';
      if PhyMdioEnxSO = '1' then
        PhyMdioxD <= PhyMdioxDO;
      end if;
    end process;
    
  end generate genPhy;


  -- process doing 4 8bit wishbone writes to access a 32 bit slave on the data bus
  process
  begin
    wait until ClkxCI'event and ClkxCI = '1' and WbStart = '1';
    CpuCycxSI <= '1';
    CpuStbxSI <= '1';
    --CpuWexSI  <= '1';
    CpuDatxDI <= WbData(7 downto 0);
    CpuAdrxDI <= WbAdr(15 downto 2) & "00";
    wait until ClkxCI'event and ClkxCI = '1' and CpuAckxSO = '1';
    CpuCycxSI <= '1';
    CpuStbxSI <= '1';
    --CpuWexSI  <= '1';
    CpuDatxDI <= WbData(15 downto 8);
    CpuAdrxDI <= WbAdr(15 downto 2) & "01";
    wait until ClkxCI'event and ClkxCI = '1' and CpuAckxSO = '1';
    CpuCycxSI <= '1';
    CpuStbxSI <= '1';
    --CpuWexSI  <= '1';
    CpuDatxDI <= WbData(23 downto 16);
    CpuAdrxDI <= WbAdr(15 downto 2) & "10";
    wait until ClkxCI'event and ClkxCI = '1' and CpuAckxSO = '1';
    CpuCycxSI <= '1';
    CpuStbxSI <= '1';
    --CpuWexSI  <= '1';
    CpuDatxDI <= WbData(31 downto 24);
    CpuAdrxDI <= WbAdr(15 downto 2) & "11";
    wait until ClkxCI'event and ClkxCI = '1' and CpuAckxSO = '1';
    CpuCycxSI <= '0';
    CpuStbxSI <= '0';
    --CpuWexSI  <= '0';
    CpuDatxDI <= x"00";
    CpuAdrxDI <= x"0000";
    WbDone    <= '1';
    wait until ClkxCI'event and ClkxCI = '1' and WbStart = '0';
    WbDone    <= '0';
  end process;

end;
