--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/08/11 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Naming convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* Test bench of the dram controller and DMA unit. This is test bench for manual signal inspection
--*
--***************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity dramCtrl_test is
end dramCtrl_test;

architecture arch_dram_test of dramCtrl_test is

  -- Component Declaration for the Unit Under Test (UUT)
  component dramCtrl
    generic(
      IS_SIMULATION : string;
      WB_BURST_LEN  : integer);
    port(
      -- system interface
      ResetSwitchxR : in    std_logic;  -- reset switch input (this comes from a push button)
      RstSocxRI     : in    std_logic;  -- logic reset for wishbone bus
      RstCorexRI    : in    std_logic;  -- logic reset for analyzer core (sampling clock domain)
      CalibDonexSO  : out   std_logic;
      -- clocks
      BoardClkxCI   : in    std_logic;  -- positive of differential on board clock (200MHz)
      --        BoardClkxCIB    : in  STD_LOGIC;     -- negative of differential on board clock (200MHz)
      RefClkxCO     : out   std_logic;  -- PLL generated reference clock for the system (25MHz)
      -- interface to sampling core
      CoreClkxC     : in    std_logic;  -- samplin system clock 100MHz
      DataxDI       : in    std_logic_vector(31 downto 0);
      WritexSI      : in    std_logic;
      -- wishbone dma master interface
      WbClkxCI      : in    std_logic;  -- wishbone clock (master & slave)
      MaCycxSO      : out   std_logic;
      MaStbxSO      : out   std_logic;
      MaWexSO       : out   std_logic;
      MaSelxDO      : out   std_logic_vector(3 downto 0);
      MaAdrxDO      : out   std_logic_vector(31 downto 0);
      MaDatxDO      : out   std_logic_vector(31 downto 0);
      MaDatxDI      : in    std_logic_vector(31 downto 0);
      MaAckxSI      : in    std_logic;
      MaErrxSI      : in    std_logic;
      -- wishbone configuration slave interface
      SlCycxSI      : in    std_logic;
      SlStbxSI      : in    std_logic;
      SlWexSI       : in    std_logic;
      SlAdrxDI      : in    std_logic_vector(4 downto 2);
      SlDatxDI      : in    std_logic_vector(31 downto 0);
      SlDatxDO      : out   std_logic_vector(31 downto 0);
      SlAckxSO      : out   std_logic;
      IntxSO        : out   std_logic;
      -- memory interface
      MemDQxDIO     : inout std_logic_vector(15 downto 0);
      MemAdrxDO     : out   std_logic_vector(12 downto 0);
      MemBaxDO      : out   std_logic_vector(2 downto 0);
      MemRasxEO     : out   std_logic;
      MemCasxEO     : out   std_logic;
      MemWexEO      : out   std_logic;
      MemOdtxO      : out   std_logic;
      MemCkexEO     : out   std_logic;
      MemClkxCO     : out   std_logic;
      MemClkxCOB    : out   std_logic;
      MemLDqsxEO    : inout std_logic;
      MemLDqsxEOB   : inout std_logic;
      MemUDqsxEO    : inout std_logic;
      MemUDqsxEOB   : inout std_logic;
      MemLdmxEO     : out   std_logic;
      MemUdmxEO     : out   std_logic;
      RzqxIO        : inout std_logic;
      ZioxIO        : inout std_logic
      );
  end component;


  component ddr2
--      Component ddr2_model_c3
    port (
      ck      : in    std_logic;
      ck_n    : in    std_logic;
      cke     : in    std_logic;
      cs_n    : in    std_logic;
      ras_n   : in    std_logic;
      cas_n   : in    std_logic;
      we_n    : in    std_logic;
      dm_rdqs : inout std_logic_vector(1 downto 0);
      ba      : in    std_logic_vector(2 downto 0);
      addr    : in    std_logic_vector(12 downto 0);
      dq      : inout std_logic_vector(15 downto 0);
      dqs     : inout std_logic_vector(1 downto 0);
      dqs_n   : inout std_logic_vector(1 downto 0);
      rdqs_n  : out   std_logic_vector(1 downto 0);
      odt     : in    std_logic
      );
  end component;

  --Inputs
  signal RstxRI      : std_logic                     := '1';
  signal CalibDonexS : std_logic;
  signal MemClkxCI   : std_logic                     := '0';
  signal MemClkxCIB  : std_logic                     := '1';
  signal RefClkxCO   : std_logic;
  signal ClkxC       : std_logic                     := '0';
  signal DataxDI     : std_logic_vector(31 downto 0) := (others => '0');
  signal WritexSI    : std_logic                     := '0';

  --BiDirs
  signal MemDQxDIO   : std_logic_vector(15 downto 0);
  signal MemLDqsxEO  : std_logic;
  signal MemLDqsxEOB : std_logic;
  signal MemUDqsxEO  : std_logic;
  signal MemUDqsxEOB : std_logic;
  signal RzqxIO      : std_logic;
  signal ZioxIO      : std_logic;

  --Outputs
  signal RstxRO       : std_logic;
  signal DataxDO      : std_logic_vector(31 downto 0);
  signal DataReadyxSO : std_logic;
  signal MemAdrxDO    : std_logic_vector(12 downto 0);
  signal MemBaxDO     : std_logic_vector(2 downto 0);
  signal MemRasxEO    : std_logic;
  signal MemCasxEO    : std_logic;
  signal MemWexEO     : std_logic;
  signal MemOdtxO     : std_logic;
  signal MemCkexEO    : std_logic;
  signal MemClkxCO    : std_logic;
  signal MemClkxCOB   : std_logic;
  signal MemLdmxEO    : std_logic;
  signal MemUdmxEO    : std_logic;

  -- Wishbone signals
  signal WbClkxC  : std_logic                     := '0';
  signal MaCycxSO : std_logic;
  signal MaStbxSO : std_logic;
  signal MaWexSO  : std_logic;
  signal MaSelxDO : std_logic_vector(3 downto 0);
  signal MaAdrxDO : std_logic_vector(31 downto 0);
  signal MaDatxDO : std_logic_vector(31 downto 0);
  signal MaDatxDI : std_logic_vector(31 downto 0) := (others => '0');
  signal MaAckxSI : std_logic                     := '0';
  signal MaErrxSI : std_logic                     := '0';
  -- wishbone configuration slave interface
  signal SlCycxSI : std_logic                     := '0';
  signal SlStbxSI : std_logic                     := '0';
  signal SlWexSI  : std_logic                     := '0';
  signal SlAdrxDI : std_logic_vector(4 downto 0)  := (others => '0');
  signal SlDatxDI : std_logic_vector(31 downto 0) := (others => '0');
  signal SlDatxDO : std_logic_vector(31 downto 0);
  signal SlAckxSO : std_logic;
  signal IntxSO   : std_logic;

  -- control signal for modelsim to stop simulation at end of testbench
  signal TBDone : std_logic := '0';

  signal dqs   : std_logic_vector(1 downto 0);
  signal dqs_n : std_logic_vector(1 downto 0);
  signal dm    : std_logic_vector(1 downto 0);
  
begin

  -- Instantiate the Unit Under Test (UUT)
  uut : dramCtrl
    generic map (
      IS_SIMULATION => "TRUE",
      WB_BURST_LEN  => 4
      )
    port map (
      -- system interface
      ResetSwitchxR => RstxRI,  -- reset switch input (this comes from a push button)
      RstSocxRI     => RstxRI,          -- logic reset for wishbone bus
      RstCorexRI    => RstxRI,  -- logic reset for analyzer core (sampling clock domain)
      CalibDonexSO  => CalibDonexS,
      BoardClkxCI   => MemClkxCI,
      --    BoardClkxCIB        => MemClkxCIB,
      RefClkxCO     => RefClkxCO,
      -- interface to sampling core
      CoreClkxC     => ClkxC,
      DataxDI       => DataxDI,
      WritexSI      => WritexSI,
      -- wishbone dma master interface
      WbClkxCI      => WbClkxC,
      MaCycxSO      => MaCycxSO,
      MaStbxSO      => MaStbxSO,
      MaWexSO       => MaWexSO,
      MaSelxDO      => MaSelxDO,
      MaAdrxDO      => MaAdrxDO,
      MaDatxDO      => MaDatxDO,
      MaDatxDI      => MaDatxDI,
      MaAckxSI      => MaAckxSI,
      MaErrxSI      => MaErrxSI,
      -- wishbone configuration slave interface
      SlCycxSI      => SlCycxSI,
      SlStbxSI      => SlStbxSI,
      SlWexSI       => SlWexSI,
      SlAdrxDI      => SlAdrxDI(4 downto 2),
      SlDatxDI      => SlDatxDI,
      SlDatxDO      => SlDatxDO,
      SlAckxSO      => SlAckxSO,
      IntxSO        => IntxSO,

      MemDQxDIO   => MemDQxDIO,
      MemAdrxDO   => MemAdrxDO,
      MemBaxDO    => MemBaxDO,
      MemRasxEO   => MemRasxEO,
      MemCasxEO   => MemCasxEO,
      MemWexEO    => MemWexEO,
      MemOdtxO    => MemOdtxO,
      MemCkexEO   => MemCkexEO,
      MemClkxCO   => MemClkxCO,
      MemClkxCOB  => MemClkxCOB,
      MemLDqsxEO  => MemLDqsxEO,
      MemLDqsxEOB => MemLDqsxEOB,
      MemUDqsxEO  => MemUDqsxEO,
      MemUDqsxEOB => MemUDqsxEOB,
      MemLdmxEO   => MemLdmxEO,
      MemUdmxEO   => MemUdmxEO,
      RzqxIO      => RzqxIO,
      ZioxIO      => ZioxIO
      );
-- elpida memory model          
--      mem :  ede1116
--      PORT MAP (
--              cas_n => MemCasxEO,
--              ck => MemClkxCO,
--              ck_n => MemClkxCOB,
--              cke => MemCkexEO,
--              cs_n => '0',
--              ldm => MemLdmxEO,
--              odt => MemOdtxO,
--              ras_n => MemRasxEO,
--              udm => MemUdmxEO,
--              we_n => MemWexEO,
--              a => MemAdrxDO,
--              ba => MemBaxDO,
--              ldqs => MemLDqsxEO,
--              ldqs_n => MemLDqsxEOB,
--              udqs => MemUDqsxEO,
--              udqs_n => MemUDqsxEOB,
--              dq => MemDQxDIO
--      );
  
  -- micron memory model
--      mem : ddr2_model_c3
  mem : ddr2
    port map (
      ck      => MemClkxCO,
      ck_n    => MemClkxCOB,
      cke     => MemCkexEO,
      cs_n    => '0',
      ras_n   => MemRasxEO,
      cas_n   => MemCasxEO,
      we_n    => MemWexEO,
      dm_rdqs => dm,
      ba      => MemBaxDO,
      addr    => MemAdrxDO,
      dq      => MemDQxDIO,
      dqs     => dqs,
      dqs_n   => dqs_n,
      rdqs_n  => open,
      odt     => MemOdtxO

      );

  dqs   <= MemUDqsxEO & MemLDqsxEO;
  dqs_n <= MemUDqsxEO & MemLDqsxEO;
  dm    <= MemUdmxEO & MemLdmxEO;

  -- 100MHz board clock
  boardClk : process
  begin
    MemClkxCI  <= '0';
    MemClkxCIB <= '1';
    wait for 5 ns;
    MemClkxCI  <= '1';
    MemClkxCIB <= '0';
    wait for 5 ns;
  end process;

  -- 100MHz sampling core clock
  sysClk : process
  begin
    ClkxC <= '0';
    wait for 5 ns;
    ClkxC <= '1';
    wait for 5 ns;
  end process;

  -- 40MHz wishbone clock
  wbClk : process
  begin
    WbClkxC <= '0';
    wait for 12.5 ns;
    WbClkxC <= '1';
    wait for 12.5 ns;
  end process;

  -- generate ack signal for the wb master
--      wbAckGen : process (WbClkxC)
--      begin
--              if WbClkxC'event and WbClkxC='1' then
--                      if MaCycxSO='1' and MaStbxSO='1' and MaAckxSI='0' then
--                              MaAckxSI <= '1';
--                      else
--                              MaAckxSI <= '0';
--                      end if;
--              end if; 
--      end process;
  MaAckxSI <= MaCycxSO and MaStbxSO;

  -- Stimulus process
  stim_proc : process
  begin
    -- generate reset
    wait for 1 us;
    RstxRI <= '0';
    -- wait until calibartion is done
    --wait for 500 us;
    --wait until ClkxC'event and ClkxC='1';       
    wait until RstxRO = '0';
    wait until CalibDonexS = '1';

    -- create a write 
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"01020304";
    WritexSI <= '1';
    -- create a write
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"05060708";
    WritexSI <= '1';
    -- create a write
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"090a0b0c";
    WritexSI <= '1';
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"00000000";
    WritexSI <= '0';
    -- create a write
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"0d0e0f10";
    WritexSI <= '1';
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"00000000";
    WritexSI <= '0';
    -- create a write
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"11121314";
    WritexSI <= '1';
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"00000000";
    WritexSI <= '0';
    -- create a write
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"15161718";
    WritexSI <= '1';
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"00000000";
    WritexSI <= '0';
    -- create a write
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"191a1b1c";
    WritexSI <= '1';
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"00000000";
    WritexSI <= '0';
    -- create a write
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"1d1e1f20";
    WritexSI <= '1';
    wait until ClkxC'event and ClkxC = '1';
    DataxDI  <= x"00000000";
    WritexSI <= '0';

    wait for 100 ns;

    -- set up an dma transfer

    -- set config flags
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '1';
    SlAdrxDI <= "00000";
    SlDatxDI <= x"0000000E";  -- E means clear error and done flag, set interrupt enable flag
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    -- set src address
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '1';
    SlAdrxDI <= "00100";
    SlDatxDI <= x"07fffff0";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    -- set destination address
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '1';
    SlAdrxDI <= "01000";
    SlDatxDI <= x"00008000";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    -- length
    -- set length
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '1';
    SlAdrxDI <= "01100";
    SlDatxDI <= x"00000008";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    wait for 50 ns;

    -- read length
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '0';
    SlAdrxDI <= "01100";
    SlDatxDI <= x"00000000";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    wait until IntxSO = '1';
    wait for 100 ns;

    -- set config flags to clear interrupt
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '1';
    SlAdrxDI <= "00000";
    SlDatxDI <= x"0000000E";  -- E means clear error and done flag, set interrupt enable flag
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    -- read config flags
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '0';
    SlAdrxDI <= "00000";
    SlDatxDI <= x"00000000";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    -- read destination address
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '0';
    SlAdrxDI <= "01000";
    SlDatxDI <= x"00000000";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    -- read source address
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '0';
    SlAdrxDI <= "00100";
    SlDatxDI <= x"00000000";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    -- read the memory write address
    wait until WbClkxC'event and WbClkxC = '1';
    SlCycxSI <= '1';
    SlStbxSI <= '1';
    SlWexSI  <= '0';
    SlAdrxDI <= "10000";
    SlDatxDI <= x"00000000";
    wait until WbClkxC'event and WbClkxC = '1' and SlAckxSO = '1';
    SlCycxSI <= '0';
    SlStbxSI <= '0';
    SlWexSI  <= '0';

    wait for 200 ns;

    TBDone <= '1';

    wait;
  end process;

end;
